#Copyright (C) 2010 Unai Zalakain De Graeve
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

class Board:
    tag = 'board'
    fill = '#777777'

class Holes:
    tag = 'hole'
    fill = '#FFFFFF'
    outline = '#FFFFFF'

class Marbles:
    tag = 'marble'
    selected = '#555555'

class Players:
    class Black(int):
        fill = '#000000'
        vp = 0
        positions = [
                       (3,3), (3,4), (3,5),
             (2,1), (2,2), (2,3), (2,4), (2,5), (2,6),
                (1,1), (1,2), (1,3), (1,4), (1,5),
                ]

    class White(int):
        fill = '#FFFFFF'
        vp = 3
        positions = [
                (9,5), (9,6), (9,7), (9,8), (9,9),
            (8,4), (8,5), (8,6), (8,7), (8,8), (8,9),
                      (7,5), (7,6), (7,7),
                ]



#Copyright (C) 2010 Unai Zalakain De Graeve
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

'Helper classes and functions.'

from math import sqrt, cos, sin

class NoNegIndexList(list):
    '''NoNegIndexList(iter) -> list that raises IndexError on
    negative indexes.'''

    def __getitem__(self, key):
        if key < 0:
            raise IndexError
        return super(NoNegIndexList, self).__getitem__(key)

class Reductors:
    @staticmethod
    def equal(items):
        '''equal(items) -> return True if all the items are equal,
        False otherways.'''
        def equal(a, b):
            if all((a, b)) and a == b:
                return a
            return False
        return reduce(equal, items)

    @staticmethod
    def consec(items):
        '''consec(items) -> return True if items are a, a+1, a+2, ...'''
        def consec(a, b):
            if all((a, b)) and a + 1 == b:
                return b
            return False
        return reduce(consec, items)

class C(list):
    '''C(x, y) -> list with x and y attributes set to the two arguments,
    ideal for coordinates.'''
    def __init__(self, x, y):
        super(C, self).__init__((x,y))
        self.x = x
        self.y = y 

def circle(coord, diameter):
    '''circle(coordinate, diameter) -> calculate the top left
    and the bottom right coordinates of an circle.

    coordinate -> center of the circle, C instance.
    diameter -> diameter of the circle.'''

    r = diameter / 2
    a = C(coord.x - r, coord.y + r)
    b = C(coord.x + r, coord.y - r)
    return a,b

def get_coord(center, edge, pos, radian):
    '''get_coord(center, edge, pos, radian) -> get the coordinate
    of pos in an abalone table.

    center -> center of the table, C instance.
    edge -> longitude of the edges of the board.
    pos -> position, a (row, column) tuple.
    radian -> spinning radian.'''
    def pc(row, column):
        y = (row - 1)
        x = (column - 1) - y * 1/2.0
        #get the separation between column and rows
        column_sep = edge / 5.0
        row_sep = sqrt(3/4.0 * column_sep ** 2)
        return c.x + (x - 2.0) * column_sep, c.y - (y - 4.0) * row_sep

    c = center
    p = C(*pc(*pos))
    a = radian

    x = cos(a) * (p.x - c.x) - sin(a) * (p.y - c.y) + c.x
    y = sin(a) * (p.x - c.x) + cos(a) * (p.y - c.y) + c.y
    return C(x, y)



#Copyright (C) 2010 Unai Zalakain De Graeve
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

'''Tkinter interface for abalone.'''

import abalone
from library import C, circle, get_coord
import config

from Tkinter import Tk, Canvas, Frame, Button
from Tkinter import TOP, BOTTOM, X, BOTH
from math import pi, sqrt

from os.path import join, dirname
PATH = dirname(__file__)

class Group(abalone.Group):
    '''Nothing, for the moment.'''
    pass

#load the players conf
Players = {
        abalone.BLACK: config.Players.Black(abalone.BLACK),
        abalone.WHITE: config.Players.White(abalone.WHITE),
        }

class Board(Canvas):
    '''Board(*args, **kwargs) -> Abalone board in Tkinter Canvas.
    
    *args, **kwargs -> arguments for Tkinter.Canvas'''

    matrix = abalone.Matrix()
    vps = [ d * pi / 180 for d in range(360, 0, -60) ]
    selected = Group()

    def __init__(self, *args, **kwargs):
        Canvas.__init__(self, *args, **kwargs)
        self.bind('<Button-1>', self.update_selection)

    def resize(self):
        '''resize() -> resize all the canvas elements and draw them.'''
        self.update_size()
        self.draw_board()
        self.draw_holes()
        self.draw_marbles()

    def update_size(self):
        '''update_size() -> update the size of the board.'''
        #get the canvas size and his center
        self.size = C(self.winfo_width(), self.winfo_height())
        self.center = C(*[ c/2.0 for c in self.size ])

        #if the canvas width is smaller that his height, the edge is the width/2
        if self.size.x < self.size.y:
            self.edge = self.size.x / 2.0
            #get the side from the edge
            self.side = sqrt(3/4.0 * self.edge ** 2) 
        #else, the side is height/2
        else:
            self.side = self.size.y / 2.0
            #get the edge from the size
            self.edge = sqrt(4/3.0 * self.side ** 2)

    def draw_board(self):
        '''draw_board() -> draw the board.'''
        #get the right coords and then make the simmetry with the left ones
        right = [(self.edge/2.0, self.side), (self.edge, 0), (self.edge/2.0, -self.side)]
        left = [ (-x, y) for (x, y) in right ]

        #center the coords, convert them to integers and unpack them
        coords = []
        [ (coords.append(int(self.center.x + x)), coords.append(int(self.center.y + y))) for (x,y) in right + left[::-1] ]

        #draw the polygon
        self.delete(config.Board.tag)
        self.create_polygon(
                *coords, 
                fill=config.Board.fill, 
                tags=config.Board.tag)
    
    def draw_holes(self):
        '''draw_holes() -> draw board holes.'''
        self.delete(config.Holes.tag)
        for pos in self.matrix:
            t, b = circle(self.get_coord(pos), self.edge / 9)
            self.create_oval(
                    t.x, t.y, b.x, b.y, 
                    fill=config.Holes.fill, 
                    outline=config.Holes.outline,
                    tags=config.Holes.tag)

    def draw_marbles(self):
        '''draw_marbles() -> draw the marbles.'''
        self.selected = Group()
        self.delete(config.Marbles.tag)
        for marble in self.master.marbles:
            t, b = circle(self.get_coord(marble['position']), self.edge / 8)
            marble.box = t, b
            marble.id = self.create_oval(
                    t.x, t.y, b.x, b.y, 
                    fill=Players[marble['owner']].fill, 
                    tags=config.Marbles.tag)

    def update_selection(self, e):
        '''update_selection(event) -> update self.selected with the marble
        on coordinates event.x and event.y, if any.'''
        def select(marble):
            self.itemconfigure(marble.id, fill=config.Marbles.selected)
            return marble
        def deselect(marble):
            self.itemconfigure(marble.id, fill=Players[marble['owner']].fill)
            return marble

        for marble in self.master.marbles:
            t, b = marble.box
            if t.x < e.x < b.x and t.y > e.y > b.y:
                if marble in self.selected:
                    self.selected.remove(deselect(marble))
                    if not self.selected.is_valid():
                        [ deselect(marble) for marble in self.selected ]
                        self.selected = Group([])
                else:
                    assert marble['owner'] == self.master.current
                    self.selected = Group(self.selected + [marble])
                    if self.selected.is_valid():
                        select(marble)
                    else:  
                        self.selected.remove(marble)
        
    def get_coord(self, pos):
        '''get_coord(position) -> get coordinates for the given position.'''
        return get_coord(self.center, self.edge, pos, self.vps[self.master.current.vp])
   

class Movement(Frame):
    '''Movement(*args, **kwargs) -> Frame with movement buttons for
    the marbles in the canvas.

    *args, **kwargs -> arguments for Tkinter.Frame.'''
    def __init__(self, *args, **kwargs):
        Frame.__init__(self, *args, **kwargs)

        Button(self, command=lambda: self.master.move(0), bitmap='@' + join(PATH, 'images/0.xbm')).grid(row=0, column=1)
        Button(self, command=lambda: self.master.move(1), bitmap='@' + join(PATH, 'images/1.xbm')).grid(row=1, column=1)
        Button(self, command=lambda: self.master.move(2), bitmap='@' + join(PATH, 'images/2.xbm')).grid(row=2, column=1)
        Button(self, command=lambda: self.master.move(3), bitmap='@' + join(PATH, 'images/3.xbm')).grid(row=2, column=0)
        Button(self, command=lambda: self.master.move(4), bitmap='@' + join(PATH, 'images/4.xbm')).grid(row=1, column=0)
        Button(self, command=lambda: self.master.move(5), bitmap='@' + join(PATH, 'images/5.xbm')).grid(row=0, column=0)

class ViewPoint(Frame):
    '''ViewPoint(*args, **kwargs) -> Frame with buttons for updating
    the viewpoint of the board for the current player.

    *args, **kwargs -> arguments for Tkinter.Frame.'''
    def __init__(self, *args, **kwargs):
        Frame.__init__(self, *args, **kwargs)

        def slc(d):
            self.master.current.vp = (self.master.current.vp + d) % 6

        Button(self,
                command=lambda: (slc(1), self.master.board.draw_marbles()),
                text='left',
                ).grid(row=0, column=0)

        Button(self,
                command=lambda: (slc(-1), self.master.board.draw_marbles()),
                text='right',
                ).grid(row=0, column=1)

class Game(abalone.Game, Tk):
    '''Game() -> new Tkinter interface for abalone.'''
    def __init__(self):
        super(Game, self).__init__()

        Tk.__init__(self)
        self.title = 'Abalone'
        self.bind('<Configure>', self.resize)
        self.minsize(400, 400)
        self.maxsize(1000, 1000)
        
    def resize(self, e):
        '''resize(event) -> update the board size
        and resize it with event.x and event.y.'''
        try:
            self.board.configure(width=self.winfo_width(), height=self.winfo_height())
            self.board.resize()
        except AttributeError:
            pass

    def start(self, *args, **kwargs):
        '''start(black=(), white=()) -> start a new abalone game.

        black -> positions for the black marbles.
        white -> positions for the white marbles.'''
        super(Game, self).start(*args, **kwargs)
        self.current = Players[self.current]

        self.board = Board(master=self, width=self.winfo_reqwidth(), height=self.winfo_reqheight())
        self.movement = Movement(master=self)
        self.viewpoint = ViewPoint(master=self)

        self.movement.pack(side=BOTTOM, fill=X)
        self.viewpoint.pack(side=BOTTOM, fill=X)
        self.board.pack(side=TOP, fill=BOTH)
        self.board.resize()

    def next(self):
        '''next() -> change the current player.'''
        if self.current == abalone.BLACK:
            self.current = Players[abalone.WHITE]
        else:
            self.current = Players[abalone.BLACK]

    def move(self, direction):
        '''move(direction) -> move the marbles selected in board in direction.
        
        direction -> direction of movement in range(6).'''
        direction = (direction + self.current.vp) % 6
        super(Game, self).move(self.board.selected, direction)

        self.next()
        looser = self.get_looser()
        if looser:
            print looser
            self.stop()
        self.board.draw_marbles()

    def stop(self):
        '''stop() -> destroy the current game.'''
        self.board.destroy()
        self.movement.destroy()
        self.viewpoint.destroy()




#!/usr/bin/env python

from distutils.core import setup

setup(
        name='abalone',
        version='0.1',

        author='Unai Zalakain De Graeve',
        author_email='zalaka@gmail.com',
        url='http://lagunak.gisa-elkartea.org/projects/abalone',
        license='GPL v3',

        description='Abalone strategy game',
        classifiers=[
            'Development Status :: 4 - Beta',
            'Environment :: X11 Applications',
            'Intended Audience :: End Users/Desktop',
            'License :: OSI Approved :: GNU General Public License (GPL)',
            'Natural Language :: English',
            'Programming Language :: Python',
            'Topic :: Games/Entertainment :: Board Games',
            'Topic :: Games/Entertainment :: Turn Based Strategy',
            ],

        packages=['abalone'],
        package_data={'abalone': ['images/*']},
        scripts=['scripts/abalone'],
        requires=['Tkinter'],
        )



#Copyright (C) 2010 Unai Zalakain De Graeve
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

'''Game logic of Abalone'''

from operator import add, sub
from gettext import gettext as _

from library import NoNegIndexList, Reductors

#representation of teams.
#NOTE: they can't be 0.
BLACK = 1
WHITE = -1

class Matrix(list):
    '''Matrix() -> list with all the positions of an abalone's board.'''

    def __init__(self):
        rows = range(1, 10)
        ranges = zip([1] * 5 + range(2, 6), range(6, 10) + [10] * 5)

        self.rows = NoNegIndexList(rows)
        self.columns = NoNegIndexList(rows)

        rows = iter(rows)
        for r in ranges:
            row = rows.next()
            self.extend([ (row, column) for column in range(*r) ])

class Marble(dict):
    '''Marble() -> dict with 'position' and 'owner' 
    keys that represents an abalone's marble.'''

    def __init__(self, position, owner):
        self['position'] = position
        self['owner'] = owner

class MarbleManager(list):
    '''MarbleManager(Marbles) -> list of Marble objects with special
    methods get_pos and get_owner.'''

    def get_pos(self, positions):
        '''get_pos(positions) -> get the Marble objects that have their
        position key in positions.
        
        positions -> list of (row, column) tuples.'''
        return [ marble for marble in self if marble['position'] in positions ]

    def get_owner(self, owner):
        '''get_owner(owner) -> get the Marble objects that have their
        owner key == owner.
        
        owner -> BLACK or WHITE'''
        return [ marble for marble in self if marble['owner'] == owner ]

class Group(list):
    '''Group(Marbles) -> list of Marble objects with special methods
    update and is_valid.'''

    def __init__(self, marbles=[]):
        assert all([ isinstance(marble, Marble) for marble in marbles ]), self.__doc__
        self.extend(sorted(marbles, key=lambda marble: marble['position']))

    def update(self, new):
        '''update(updated_marbles) -> update the position attribute of
        the marbles in the Group with the position attribute of others.
        
        updated_marbles -> updated marbles to get the position attr.'''
        for old_m, new_m in zip(self, new):
            old_m['position'] = new_m['position']

    def is_valid(self):
        '''is_valid() -> return bool saying if this Group is a valid one.
        If valid, set "owner" and "positions" attributes with the owner and
        the positions of the marbles.'''

        #must have between 1 and 3 members
        if not 1 <= len(self) <= 3:
            return False

        #the owner of all the marbles must be the same
        owners = [marble['owner'] for marble in self ]
        if not Reductors.equal(owners):
            return False
        self.owner = owners[0]

        #the position of the marbles must be in line
        positions = [ marble['position'] for marble in self ]
        if not len(positions) == 1:
            rows = [ pos[0] for pos in positions ]
            columns = [ pos[1] for pos in positions ]
            if not any((
                    Reductors.equal(rows) and Reductors.consec(columns),
                    Reductors.consec(rows) and Reductors.equal(columns),
                    Reductors.consec(rows) and Reductors.consec(columns))):
                return False
        self.positions = positions

        return True

class Logic(Matrix):
    '''Logic() -> group of methods with the game logic of abalone.'''
    marbles = []

    def is_in_matrix(self, group):
        '''is_in_matrix(group) -> return True if group is in Matrix,
        False otherways.
        
        group -> Group instance.'''
        return all([ marble['position'] in self for marble in group ])

    def get_moved(self, group, direction):
        '''get_moved(group, direction) -> return the group moved in direction.
        
        group -> Group instance.
        direction -> direction of movement, in range(6)'''
        diffs = 1, 1, 0, -1, -1, 0
        try:
            row_diff = diffs[direction + 1]
        except IndexError:
            row_diff = diffs[0]
        column_diff = diffs[direction]

        l = []
        for marble in group:
            try:
                row = self.rows[self.rows.index(marble['position'][0]) + row_diff]
                column = self.columns[self.columns.index(marble['position'][1]) + column_diff]
            except IndexError:
                pass
            else:
                if (row, column) in self:
                    l.append(Marble((row, column), marble['owner']))
        return Group(l)

    def get_obstacles(self, group, direction):
        '''get_obstacles(group, direction) -> return the obstacles when trying to move
        group in direction.
        
        group -> Group instance.
        direction -> direction of movement, in range(6).'''
        moved_group = self.get_moved(group, direction)
        diff = [ marble['position'] for marble in moved_group if marble not in group ]
        return Group([ marble for marble in self.marbles if marble['position'] in diff ])

    def is_lateral_move(self, group, direction):
        '''is_lateral_move(group, direction) -> return True if the movement
        of group in direction is a lateral move, False otherways.
        
        group -> Group instance.
        direction -> direction of movement, in range(6).'''
        moved_group = self.get_moved(group, direction)
        return all([ marble not in group for marble in moved_group ])

    def get_mirror_obstacles(self, group, direction):
        '''get_mirror_obstacles(group, direction) -> return the obstacles when
        trying to move group in direction len(group) times.
        
        group -> Group instance.
        direction -> direction of movement, in range(6).'''
        obstacles = []
        for movement in range(len(group)):
            obstacle = self.get_obstacles(group, direction)
            if not obstacle:
                break
            obstacles.extend(obstacle)
            group = self.get_moved(group, direction)
        return Group(obstacles)

    def is_pushable(self, group, mirror_obstacles):
        '''is_pushable(group, mirror_obstacles) -> return True if group
        can push the mirror_obstacles, False otherways.
        
        group -> Group instance
        mirror_obstacles -> obstacles returned by Logic.get_mirror_obstacles'''
        return all((
                mirror_obstacles.is_valid(),
                self.is_in_matrix(mirror_obstacles),
                len(group) > len(mirror_obstacles),
                ))

class Game(object):
    '''Game() -> An abalone game.'''

    logic = Logic()

    def start(self, black=(), white=()):
        '''start(black=(), white=()) -> start a new game. 
        
        black -> positions of the marbles of the black team.
        white -> positions of the marbles of the white team.'''
        self.marbles = MarbleManager(
                [ Marble(position, BLACK) for position in black ] + \
                [ Marble(position, WHITE) for position in white ]
                )
        self.current = BLACK
        self.initial = len(black), len(white)

    def get_looser(self):
        '''get_looser() -> get the looser team, False if no one.'''
        for team, initial in zip((BLACK, WHITE), self.initial):
            if initial - len(self.marbles.get_owner(team)) >= initial / (14/6.0):
                return team
        return False

    def next(self):
        '''next() -> switch the actual player.'''
        if self.current == BLACK:
            self.current = WHITE
        else:
            self.current = BLACK

    def move(self, positions_or_group, direction):
        '''move(positions_or_group, direction) -> move the positions_or_group
        in direction. raises AssertionError if the move isn't possible.

        positions_or_group -> position of the marbles of a Group instance with
        them.

        direction -> direction of the movement, in range(6).'''

        if isinstance(positions_or_group, Group):
            group = positions_or_group
        else:
            group = Group(self.marbles.get_pos(positions_or_group))

        self.logic.marbles = self.marbles

        assert group.is_valid() and self.logic.is_in_matrix(group), _('The group of marbles isn\'t valid.')
        assert group.owner == self.current, _('The marbles aren\'t yours.')

        obstacles = self.logic.get_obstacles(group, direction)

        if not obstacles:
            moved_group = self.logic.get_moved(group, direction)

        else: 
            assert not self.logic.is_lateral_move(group, direction), _('You can\'t push an enemy in a lateral move.')
            assert obstacles.is_valid()
            assert obstacles.owner is not self.current, _('You can\'t push your own marbles.')

            enemy = self.logic.get_mirror_obstacles(group, direction)
            assert self.logic.is_pushable(group, enemy), _('You can\'t push the enemy.')

            moved_enemy = self.logic.get_moved(enemy, direction)
            if len(enemy) > len(moved_enemy):
                self.marbles.remove(enemy[-1])
                enemy.pop(-1)
            enemy.update(moved_enemy)

            moved_group = self.logic.get_moved(group, direction)

        assert len(group) == len(moved_group) and moved_group.is_valid() and self.logic.is_in_matrix(moved_group), _('You can\'t move there.')
        group.update(moved_group)



#Copyright (C) 2010 Unai Zalakain De Graeve
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest

from abalone import Marble, Group, Game
class GameLogic(unittest.TestCase):

    def setUp(self):
        self.game = Game()

    def test_valid_groups(self):
        '''Assert that the groups of 1/2/3 marbles are valid.'''

        groups = (
                [(5,5)],
                [(5,5), (5,6)],
                [(5,5), (5,6), (5,7)],
                )
        for group in groups:
            self.game.start(group)
            group = Group(self.game.marbles.get_pos(group))
            self.assert_(group.is_valid())

    def test_group_ordering(self):
        '''Assert that the marbles of a group are ordered by their position'''

        group = [(2,2), (3,3), (1,1)]
        ordered = [(1,1), (2,2), (3,3)]

        group = Group([ Marble(pos, 1) for pos in group ])
        self.assert_(group.is_valid())
        self.assert_(group.positions == ordered)


    def test_invalid_groups(self):
        '''Assert that the groups with any of this characteristics
        is invalid:
            - Has not between 1 and 3 members.
            - Isn't in line.
            - Has invalid positions.
            - Has repeated positions.
            - There're enemys marbles in it.
        '''

        groups = (
                [], #no suficient members
                [(5,5), (5,6), (5,7), (5,8)], #too many members
                [(5,5), (5,7), (5,8)], #not inline
                [(10,10), (5,5)], #invalid positions
                [(5,5), (5,5), (6,6)], #repeated positions
                )

        groups_e = (
                [(5,5), (5,6)], #one team
                [(5,5), (5,7)], #the other (the enemy)
                )
        
        for group in groups:
            self.game.start(group)
            group = Group(self.game.marbles.get_pos(group))
            self.assert_(group.is_valid() == False)

        self.game.start(groups_e[0], groups_e[1])
        group = Group(self.game.marbles.get_pos(groups_e[0] + groups_e[1]))
        self.assert_(group.is_valid() == False)

    def test_free_move(self):
        '''Assert that a group of 1/2/3 marbles can move freelly
        in the center of the board in all directions, back to their
        initial position.'''

        groups = (
                [(5,5)],
                [(5,5), (5,6)],
                [(5,5), (5,6), (5,7)],
                )

        for group in groups:
            self.game.start(group)
            group = Group(self.game.marbles.get_pos(group))
            for direction in range(6):
                self.game.move(group, direction)
                self.assert_(group.is_valid())
            self.assert_(group.positions in groups)

    def test_out_of_matrix(self):
        '''Assert that a group can't move out of the matrix.'''

        groups = (
                [(1,1)],
                [(1,1), (1,2)],
                [(1,1), (1,2), (1,3)],
                )

        for group in groups:
            self.game.start(group)
            self.assertRaises(AssertionError, self.game.move, group, 2)
            self.assertRaises(AssertionError, self.game.move, group, 3)

    def test_is_pushable(self):
        '''Assert that a group can push an other with less members and
        that both are moved once.'''

        blacks = (
                [(1,1), (2,2)],
                [(1,1), (2,2), (3,3)],
                [(1,1), (2,2), (3,3)],
                )

        whites = (
                [(3,3)],
                [(4,4)],
                [(4,4), (5,5)],
                )

        black_results = (
                [(2,2), (3,3)],
                [(2,2), (3,3), (4,4)],
                [(2,2), (3,3), (4,4)],
                )

        white_results = (
                [(4,4)],
                [(5,5)],
                [(5,5), (6,6)],
                )

        for b, w, br, wr in zip(blacks, whites, black_results, white_results):
            self.game.start(b, w)
            black = Group(self.game.marbles.get_pos(b))
            white = Group(self.game.marbles.get_pos(w))
            self.game.move(black, 0)
            self.assert_(black.is_valid())
            self.assert_(white.is_valid())
            self.assert_(black.positions == br)
            self.assert_(white.positions == wr)

    def test_is_not_pushable(self):
        '''Assert that you can't push an enemy if:
            - Your group hasn't more marbles.
            - There's a marble that it's yours after the enemy.
            - Your movement is lateral.
        '''

        blacks = (
                [(1,1), (2,2), (3,3)],
                [(1,1), (2,2)],
                [(1,1)],
                [(1,1), (2,2), (3,3), (6,6)],
                [(1,1), (2,2), (3,3), (5,5)],
                [(1,1), (2,2), (4,4)],
                [(1,1), (1,2), (1,3)],
                [(1,1), (1,2)],
                )

        whites = (
                [(4,4), (5,5), (6,6)],
                [(3,3), (4,4)],
                [(2,2)],
                [(4,4), (5,5)],
                [(4,4)],
                [(3,3)],
                [(2,2), (2,3), (2,4)],
                [(2,2), (2,3)],
                )

        for black, white in zip(blacks, whites):
            self.game.start(black, white)
            self.assertRaises(AssertionError, self.game.move, black, 0)

    def test_killed(self):
        '''Assert that if you push the enemy out of the matrix,
        his marble gets killed.'''

        black = [(4,4), (4,5), (4,6)]
        white = [(4,7), (4,8)]

        black_result = [(4,5), (4,6), (4,7)]
        white_result = [(4,8)]

        self.game.start(black, white)
        black = Group(self.game.marbles.get_owner(1))
        white = Group(self.game.marbles.get_owner(-1))
        self.game.move(black, 1)
        black = Group(self.game.marbles.get_owner(1))
        white = Group(self.game.marbles.get_owner(-1))
        self.assert_(black.is_valid())
        self.assert_(white.is_valid())
        self.assert_(black.positions == black_result)
        self.assert_(white.positions == white_result)

    def test_game(self):
        '''Test a little game with only the blacks moving until the final.'''

        black = [(7,5), (5,3), (5,6), (5,7), (5,8), (5,9), (4,5),
                (4,6), (4,7), (4,8), (3,4), (3,5), (3,6), (3,7)]

        white = [(4,2), (3,1), (3,2), (2,1), (2,2), (2,3), (2,4),
                (2,5), (2,6), (1,1), (1,2), (1,3), (1,4), (1,5)]

        movements = (
                ([(5,9), (4,8), (3,7)], 3),
                ([(5,8), (4,7), (3,6)], 3),
                ([(5,7), (4,6), (3,5)], 3),
                ([(4,8), (3,7), (2,6)], 3),
                ([(4,7), (3,6), (2,5)], 3),
                ([(4,6), (3,5), (2,4)], 3),
                )

        self.game.start(black, white)
        for position, direction in movements:
            self.game.move(position, direction)
        self.assert_(self.game.get_looser() == -1)


import tk
class Tk(unittest.TestCase):

    def setUp(self):
        self.game = tk.Game()

if __name__ == '__main__':
    unittest.main()
